@@ -1,7 +1,4 @@
 package Test::Unit::Debug;
-BEGIN {
-  $Test::Unit::Debug::VERSION = '0.25_1325'; # added by dist-tools/SetVersion.pl
-}
 
 =head1 NAME
 
@@ -102,16 +99,6 @@ sub debugged {
     return $DEBUG{$_[0] || $package};
 }
 
-
-=head1 AUTHOR
-
-Copyright (c) 2000-2002, 2005 the PerlUnit Development Team
-(see L<Test::Unit> or the F<AUTHORS> file included in this
-distribution).
-
-All rights reserved. This program is free software; you can
-redistribute it and/or modify it under the same terms as Perl itself.
-
 =head1 SEE ALSO
 
 L<Test::Unit>
