@@ -1,7 +1,4 @@
 package Test::Unit::Runner::Terminal;
-BEGIN {
-  $Test::Unit::Runner::Terminal::VERSION = '0.25_1325'; # added by dist-tools/SetVersion.pl
-}
 use strict;
 
 use base qw(Test::Unit::TestRunner); 
@@ -92,27 +89,22 @@ __END__
 
 =head1 NAME
 
-Test::Unit::Runner::Terminal - unit testing framework helper class
+Test::Unit::TestRunner - unit testing framework helper class
 
 =head1 SYNOPSIS
 
-    use Test::Unit::Runner::Terminal;
+    use Test::Unit::TestRunner;
 
-    my $testrunner = Test::Unit::Runner::Terminal->new();
+    my $testrunner = Test::Unit::TestRunner->new();
     $testrunner->start($my_test_class);
 
 =head1 DESCRIPTION
 
-This class is a test runner for the command line style use
+This class is the test runner for the command line style use
 of the testing framework.
 
-It is similar to its parent class, Test::Unit::TestRunner, but it uses
-terminal escape sequences to continually update a more informative
-status report as the runner progresses through the tests than just a
-string of dots, E's and F's.  The status report indicates the number
-of tests run, the number of failures and errors encountered, which
-test is currently being run, and where it lives in the suite
-hierarchy.
+It is used by simple command line tools like the F<TestRunner.pl>
+script provided.
 
 The class needs one argument, which is the name of the class
 encapsulating the tests to be run.
@@ -134,13 +126,18 @@ version info
 
 =head1 AUTHOR
 
-Copyright (c) 2000-2002, 2005 the PerlUnit Development Team
-(see L<Test::Unit> or the F<AUTHORS> file included in this
-distribution).
+Framework JUnit authored by Kent Beck and Erich Gamma.
+
+Ported from Java to Perl by Christian Lemburg.
+
+Copyright (c) 2000 Christian Lemburg, E<lt>lemburg@acm.orgE<gt>.
 
 All rights reserved. This program is free software; you can
 redistribute it and/or modify it under the same terms as Perl itself.
 
+Thanks go to the other PerlUnit framework people: 
+Brian Ewins, Cayte Lindner, J.E. Fritz, Zhon Johansen.
+
 =head1 SEE ALSO
 
 =over 4
